use std::error::Error;
use fixed::FixedI32;
use fixed::types::extra::U16;
use flo_curves::*;
use flo_curves::{bezier, Coord2};

use super::utility::print_info;

/*
stinkhead7ds easing curve description:

e.g. going from 120 to 140
easing function is cubic bezier going from 0.0 to 1.0

140-120=20 increments
0.0 is +0 and 1.0 is +20
so e.g. if you read a 0.5 at the given increment it's 0.5 * 20 = +10

curves are exported from reskit sampled at 1/60 increments
reskit accepts arguments for control points and exports 16.16 fixed point binary files
so a 1 second animation is 60 increments, 2 seconds is 120, etc.

each iteration be like:
1. take cubic bezier sample and post increment
2. the value at this iteration is (final - beginning) * sample
   a. so example (140-120)*0.5 for a linear curve sampled right in the middle
3. round the value to the nearest whole integer
4. do not go back to 1 if we exhausted the curve

need fixed point arithmetic multiplication
*/


/**
 * Output a binary for a cubic bezier defined by the given four points and the
 * time interval defined as number of 1/60 jiffies.
 */
pub fn get_cubic_bezier( p0: (f64, f64), c0: (f64, f64), c1: (f64, f64), p1: (f64, f64), jiffies: u16 ) -> Result<Vec<u8>, Box<dyn Error>> {
    let mut result: Vec<u8> = Vec::new();

    // 2 bytes: Number of frames this curve spans
    result.extend( jiffies.to_be_bytes() );

    let curve = bezier::Curve::from_points(
        Coord2( p0.0, p0.1 ),
        ( Coord2( c0.0, c0.1 ), Coord2( c1.0, c1.1 ) ),
        Coord2( p1.0, p1.1 )
    );

    let mut current_jiffy = 0.0;
    // (jiffies) of 16.16 fixed point integers: Each point in the curve at the given frame
    for i in 0..jiffies {
        let point = curve.point_at_pos( current_jiffy );
        let x = point.1;
        print_info( &format!( "at iteration {}, curve is {:?}, 16.16 format will be {}", i, point, x ) );
        current_jiffy += 1.0 / jiffies as f64;

        let as_fixed = FixedI32::<U16>::from_num( x );
        result.extend( as_fixed.to_be_bytes() );
    }

    Ok( result )
}